@@ -10,4 +10,14 @@ fn main() {
   {
     println!("cargo:rustc-cfg=use_colorsync_cgdisplaycreateuuidfromdisplayid");
   }
+  // link carbon hotkey on macOS
+  #[cfg(target_os = "macos")]
+  {
+    if std::env::var("CARGO_CFG_TARGET_OS").map_or(false, |os| os == "macos") {
+      println!("cargo:rustc-link-lib=framework=Carbon");
+      cc::Build::new()
+        .file("src/platform_impl/macos/carbon_hotkey/carbon_hotkey_binding.c")
+        .compile("carbon_hotkey_binding.a");
+    }
+  }
 }
