@@ -3,36 +3,33 @@
 
 /// This is a simple implementation of support for Windows Dark Mode,
 /// which is inspired by the solution in https://github.com/ysc3839/win32-darkmode
-use winapi::{
-  shared::{
-    basetsd::SIZE_T,
-    minwindef::{BOOL, DWORD, FALSE, UINT, ULONG, WORD},
-    ntdef::{LPSTR, NTSTATUS, NT_SUCCESS, PVOID, WCHAR},
-    windef::HWND,
-    winerror::S_OK,
-  },
-  um::{libloaderapi, uxtheme, winuser},
+use windows::Win32::{
+  Foundation::{BOOL, HWND, PSTR, PWSTR},
+  System::LibraryLoader::*,
+  UI::{Accessibility::*, Controls::*, WindowsAndMessaging::*},
 };
 
+use std::ffi::c_void;
+
 use crate::{platform_impl::platform::util, window::Theme};
 
 lazy_static! {
-    static ref WIN10_BUILD_VERSION: Option<DWORD> = {
+    static ref WIN10_BUILD_VERSION: Option<u32> = {
         // FIXME: RtlGetVersion is a documented windows API,
-        // should be part of winapi!
+        // should be part of win32metadata!
 
         #[allow(non_snake_case)]
         #[repr(C)]
         struct OSVERSIONINFOW {
-            dwOSVersionInfoSize: ULONG,
-            dwMajorVersion: ULONG,
-            dwMinorVersion: ULONG,
-            dwBuildNumber: ULONG,
-            dwPlatformId: ULONG,
-            szCSDVersion: [WCHAR; 128],
+            dwOSVersionInfoSize: u32,
+            dwMajorVersion: u32,
+            dwMinorVersion: u32,
+            dwBuildNumber: u32,
+            dwPlatformId: u32,
+            szCSDVersion: [u16; 128],
         }
 
-        type RtlGetVersion = unsafe extern "system" fn (*mut OSVERSIONINFOW) -> NTSTATUS;
+        type RtlGetVersion = unsafe extern "system" fn (*mut OSVERSIONINFOW) -> i32;
         let handle = get_function!("ntdll.dll", RtlGetVersion);
 
         if let Some(rtl_get_version) = handle {
@@ -48,7 +45,7 @@ lazy_static! {
 
                 let status = (rtl_get_version)(&mut vi as _);
 
-                if NT_SUCCESS(status) && vi.dwMajorVersion == 10 && vi.dwMinorVersion == 0 {
+                if status >= 0 && vi.dwMajorVersion == 10 && vi.dwMinorVersion == 0 {
                     Some(vi.dwBuildNumber)
                 } else {
                     None
@@ -86,14 +83,17 @@ pub fn try_theme(hwnd: HWND, preferred_theme: Option<Theme>) -> Theme {
     } else {
       Theme::Light
     };
-    let theme_name = match theme {
-      Theme::Dark => DARK_THEME_NAME.as_ptr(),
-      Theme::Light => LIGHT_THEME_NAME.as_ptr(),
-    };
+    let theme_name = PWSTR(
+      match theme {
+        Theme::Dark => DARK_THEME_NAME.clone(),
+        Theme::Light => LIGHT_THEME_NAME.clone(),
+      }
+      .as_mut_ptr(),
+    );
 
-    let status = unsafe { uxtheme::SetWindowTheme(hwnd, theme_name as _, std::ptr::null()) };
+    let status = unsafe { SetWindowTheme(hwnd, theme_name, PWSTR::default()) };
 
-    if status == S_OK && set_dark_mode_for_window(hwnd, is_dark_mode) {
+    if status.is_ok() && set_dark_mode_for_window(hwnd, is_dark_mode) {
       return theme;
     }
   }
@@ -116,8 +116,8 @@ fn set_dark_mode_for_window(hwnd: HWND, is_dark_mode: bool) -> bool {
   #[repr(C)]
   struct WINDOWCOMPOSITIONATTRIBDATA {
     Attrib: WINDOWCOMPOSITIONATTRIB,
-    pvData: PVOID,
-    cbData: SIZE_T,
+    pvData: *mut c_void,
+    cbData: usize,
   }
 
   lazy_static! {
@@ -128,7 +128,7 @@ fn set_dark_mode_for_window(hwnd: HWND, is_dark_mode: bool) -> bool {
   if let Some(set_window_composition_attribute) = *SET_WINDOW_COMPOSITION_ATTRIBUTE {
     unsafe {
       // SetWindowCompositionAttribute needs a bigbool (i32), not bool.
-      let mut is_dark_mode_bigbool = is_dark_mode as BOOL;
+      let mut is_dark_mode_bigbool: BOOL = is_dark_mode.into();
 
       let mut data = WINDOWCOMPOSITIONATTRIBDATA {
         Attrib: WCA_USEDARKMODECOLORS,
@@ -138,7 +138,7 @@ fn set_dark_mode_for_window(hwnd: HWND, is_dark_mode: bool) -> bool {
 
       let status = set_window_composition_attribute(hwnd, &mut data as *mut _);
 
-      status != FALSE
+      status.as_bool()
     }
   } else {
     false
@@ -154,24 +154,20 @@ fn should_apps_use_dark_mode() -> bool {
   lazy_static! {
     static ref SHOULD_APPS_USE_DARK_MODE: Option<ShouldAppsUseDarkMode> = {
       unsafe {
-        const UXTHEME_SHOULDAPPSUSEDARKMODE_ORDINAL: WORD = 132;
+        const UXTHEME_SHOULDAPPSUSEDARKMODE_ORDINAL: u16 = 132;
 
-        let module = libloaderapi::LoadLibraryA("uxtheme.dll\0".as_ptr() as _);
+        let module = LoadLibraryA("uxtheme.dll");
 
-        if module.is_null() {
+        if module.is_invalid() {
           return None;
         }
 
-        let handle = libloaderapi::GetProcAddress(
+        let handle = GetProcAddress(
           module,
-          winuser::MAKEINTRESOURCEA(UXTHEME_SHOULDAPPSUSEDARKMODE_ORDINAL),
+          PSTR(UXTHEME_SHOULDAPPSUSEDARKMODE_ORDINAL as usize as *mut _),
         );
 
-        if handle.is_null() {
-          None
-        } else {
-          Some(std::mem::transmute(handle))
-        }
+        handle.map(|handle| std::mem::transmute(handle))
       }
     };
   }
@@ -181,34 +177,23 @@ fn should_apps_use_dark_mode() -> bool {
     .unwrap_or(false)
 }
 
-// FIXME: This definition was missing from winapi. Can remove from
-// here and use winapi once the following PR is released:
-// https://github.com/retep998/winapi-rs/pull/815
-#[repr(C)]
-#[allow(non_snake_case)]
-struct HIGHCONTRASTA {
-  cbSize: UINT,
-  dwFlags: DWORD,
-  lpszDefaultScheme: LPSTR,
-}
-
-const HCF_HIGHCONTRASTON: DWORD = 1;
+const HCF_HIGHCONTRASTON: u32 = 1;
 
 fn is_high_contrast() -> bool {
   let mut hc = HIGHCONTRASTA {
     cbSize: 0,
     dwFlags: 0,
-    lpszDefaultScheme: std::ptr::null_mut(),
+    lpszDefaultScheme: PSTR::default(),
   };
 
   let ok = unsafe {
-    winuser::SystemParametersInfoA(
-      winuser::SPI_GETHIGHCONTRAST,
+    SystemParametersInfoA(
+      SPI_GETHIGHCONTRAST,
       std::mem::size_of_val(&hc) as _,
       &mut hc as *mut _ as _,
       0,
     )
   };
 
-  ok != FALSE && (HCF_HIGHCONTRASTON & hc.dwFlags) == 1
+  ok.as_bool() && (HCF_HIGHCONTRASTON & hc.dwFlags) != 0
 }
