@@ -1,6 +1,10 @@
+use parking_lot::{Mutex, MutexGuard};
 use std::{
-  char, collections::HashSet, ffi::OsString, mem::MaybeUninit, os::windows::ffi::OsStringExt,
-  sync::MutexGuard,
+  char,
+  collections::{HashMap, HashSet},
+  ffi::OsString,
+  mem::MaybeUninit,
+  os::windows::ffi::OsStringExt,
 };
 
 use windows::Win32::{
@@ -17,10 +21,13 @@ use unicode_segmentation::UnicodeSegmentation;
 use crate::{
   event::{ElementState, KeyEvent},
   keyboard::{Key, KeyCode, KeyLocation, NativeKeyCode},
-  platform_impl::platform::{
-    event_loop::ProcResult,
-    keyboard_layout::{get_or_insert_str, Layout, LayoutCache, WindowsModifiers, LAYOUT_CACHE},
-    KeyEventExtra,
+  platform_impl::{
+    platform::{
+      event_loop::ProcResult,
+      keyboard_layout::{get_or_insert_str, Layout, LayoutCache, WindowsModifiers, LAYOUT_CACHE},
+      KeyEventExtra,
+    },
+    WindowId,
   },
 };
 
@@ -37,6 +44,11 @@ pub struct MessageAsKeyEvent {
   pub is_synthetic: bool,
 }
 
+lazy_static! {
+  pub(crate) static ref KEY_EVENT_BUILDERS: Mutex<HashMap<WindowId, KeyEventBuilder>> =
+    Mutex::new(HashMap::new());
+}
+
 /// Stores information required to make `KeyEvent`s.
 ///
 /// A single Tao `KeyEvent` contains information which the Windows API passes to the application
@@ -98,7 +110,7 @@ impl KeyEventBuilder {
         }
         *result = ProcResult::Value(LRESULT(0));
 
-        let mut layouts = LAYOUT_CACHE.lock().unwrap();
+        let mut layouts = LAYOUT_CACHE.lock();
         let event_info =
           PartialKeyEventInfo::from_message(wparam, lparam, ElementState::Pressed, &mut layouts);
 
@@ -148,7 +160,7 @@ impl KeyEventBuilder {
         // At this point, we know that there isn't going to be any more events related to
         // this key press
         let event_info = self.event_info.take().unwrap();
-        let mut layouts = LAYOUT_CACHE.lock().unwrap();
+        let mut layouts = LAYOUT_CACHE.lock();
         let ev = event_info.finalize(&mut layouts.strings);
         return vec![MessageAsKeyEvent {
           event: ev,
@@ -220,7 +232,7 @@ impl KeyEventBuilder {
               return vec![];
             }
           };
-          let mut layouts = LAYOUT_CACHE.lock().unwrap();
+          let mut layouts = LAYOUT_CACHE.lock();
           // It's okay to call `ToUnicode` here, because at this point the dead key
           // is already consumed by the character.
           let kbd_state = get_kbd_state();
@@ -259,7 +271,7 @@ impl KeyEventBuilder {
       win32wm::WM_KEYUP | win32wm::WM_SYSKEYUP => {
         *result = ProcResult::Value(LRESULT(0));
 
-        let mut layouts = LAYOUT_CACHE.lock().unwrap();
+        let mut layouts = LAYOUT_CACHE.lock();
         let event_info =
           PartialKeyEventInfo::from_message(wparam, lparam, ElementState::Released, &mut layouts);
         let mut next_msg = MaybeUninit::uninit();
@@ -306,7 +318,7 @@ impl KeyEventBuilder {
   ) -> Vec<MessageAsKeyEvent> {
     let mut key_events = Vec::new();
 
-    let mut layouts = LAYOUT_CACHE.lock().unwrap();
+    let mut layouts = LAYOUT_CACHE.lock();
     let (locale_id, _) = layouts.get_current_layout();
 
     let is_key_pressed = |vk: VIRTUAL_KEY| &kbd_state[usize::from(vk)] & 0x80 != 0;
